@@ -3,12 +3,14 @@
 #include <libavcodec/avfft.h>
 #include <libavutil/audio_fifo.h>
 #include <libswresample/swresample.h>
-
-#define R_NO_REMAP
-#define STRICT_R_HEADERS
-#include <Rinternals.h>
 #include "avcompat.h"
 
+#ifdef NEW_FFT_TX_API
+#include <libavutil/tx.h>
+#endif
+
+#include <Rinternals.h>
+
 enum AmplitudeScale { AS_LINEAR, AS_SQRT, AS_CBRT, AS_LOG, NB_ASCALES };
 
 extern int total_open_handles;
@@ -23,7 +25,12 @@ typedef struct {
 typedef struct {
   uint8_t *buf;
   SwrContext *swr;
+#ifdef NEW_FFT_TX_API
+  AVTXContext *tx_ctx;
+  av_tx_fn tx_fun;
+#else
   FFTContext *fft;
+#endif
   FFTComplex *fft_data;
   AVAudioFifo *fifo;
   input_container *input;
@@ -75,7 +82,6 @@ static void close_input(input_container **x){
   input_container *input = *x;
   if(input == NULL)
     return;
-  avcodec_close(input->decoder);
   avcodec_free_context(&(input->decoder));
   avformat_close_input(&input->demuxer);
   avformat_free_context(input->demuxer);
@@ -88,8 +94,13 @@ static void close_spectrum_container(void *ptr, Rboolean jump){
   spectrum_container *s = ptr;
   if(s->input)
     close_input(&s->input);
+#ifdef NEW_FFT_TX_API
+  if(s->tx_ctx)
+    av_tx_uninit(&s->tx_ctx);
+#else
   if(s->fft)
     av_fft_end(s->fft);
+#endif
   if(s->fifo)
     av_audio_fifo_free(s->fifo);
   if(s->swr)
@@ -228,7 +239,12 @@ static SEXP run_fft(spectrum_container *output, int ascale){
   int hop_size = window_size * (1 - overlap);
   int output_range = window_size / 2;
   int iter = 0;
+#ifdef NEW_FFT_TX_API
+  float scale = 1.0f;
+  av_tx_init(&output->tx_ctx, &output->tx_fun, AV_TX_FLOAT_FFT, 0,  window_size, &scale, AV_TX_INPLACE);
+#else
   output->fft = av_fft_init(fft_bits, 0);
+#endif
   output->fft_data = av_calloc(window_size, sizeof(*output->fft_data));
   output->src_data = av_calloc(window_size, sizeof(*output->src_data));
   output->fifo = av_audio_fifo_alloc(AV_SAMPLE_FMT_FLTP, 1, window_size);
@@ -289,8 +305,12 @@ static SEXP run_fft(spectrum_container *output, int ascale){
         fft_channel[n].re = 0;
         fft_channel[n].im = 0;
       }
+#ifdef NEW_FFT_TX_API
+      output->tx_fun(output->tx_ctx, fft_channel, fft_channel, sizeof(AVComplexFloat));
+#else
       av_fft_permute(output->fft, fft_channel);
       av_fft_calc(output->fft, fft_channel);
+#endif
       output->dst_dbl = av_realloc(output->dst_dbl, round_up((iter+1) * output_range * sizeof(*output->dst_dbl)));
       for (int n = 0; n < output_range; n++) {
         FFTSample re = fft_channel[n].re;
@@ -320,8 +340,8 @@ static SEXP run_bin(spectrum_container *output){
   AVCodecContext *decoder = input->decoder;
   int max_frame_size = av_get_audio_frame_duration(decoder, 0);
   if(max_frame_size < 1){
-    REprintf("Unknown frame size for input audio format: %s\n", decoder->codec->name);
-    max_frame_size = 50000; /* Very high fallback, should hopefully never happen */
+    //REprintf("Unknown frame size for input audio format: %s\n", decoder->codec->name);
+    max_frame_size = 50000; /* Very high fallback, should rarely happen */
   }
   av_samples_alloc(&output->buf, NULL, output->channels, max_frame_size, AV_SAMPLE_FMT_S32, 0);
   int64_t elapsed = 0;
